/*
 * FILE:    GUI.java
 * AUTHORS: Kristian Hasler
 * 
 * Copyright (c) 1999 University College London
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted, for non-commercial use only, provided
 * that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Computer Science
 *      Department at University College London
 * 4. Neither the name of the University nor of the Department may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 * Use of this software for commercial purposes is explicitly forbidden
 * unless prior written permission is obtained from the authors.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
 
import java.awt.*;
import java.applet.*;
import java.util.*;

class GUI extends Panel
{
	private TextArea sessionDesc;
    private TextField[] text;
 	public Label[] label;
	public Button startButton;
    public Checkbox[] check;
	private Label sessionTitle;
	public Choice[] choice;
	private Panel panel, panelStart, panelDesc, panelInfo, panelTool;
	private Font font;
	private int fontSize;

    public GUI() {
		label=new Label[4];
		text=new TextField[4];
		choice=new Choice[4];
        check=new Checkbox[4];

		if ((System.getProperty("os.name")).startsWith("Windows")) fontSize=12;
			else fontSize=11;
		font=new Font("Helvetica", Font.PLAIN, fontSize);
		setFont(font);

		setLayout(new BorderLayout(10,10));

		// Three main panels - 1st for session info, 2nd for tools, 3rd for start button
		panelInfo=new Panel();
		panelTool=new Panel();
		panelInfo.setLayout(new BorderLayout(10,3));
		panelTool.setLayout(new FlowLayout(FlowLayout.CENTER,15,5));
		add ("North",panelInfo);
		add ("Center",panelTool);

		// Panels for session description/title and start button
		panelDesc=new Panel();
		panelStart=new Panel();
		panelDesc.setLayout(new BorderLayout());
		panelInfo.add ("North",panelDesc);
		panelInfo.add ("South",panelStart);

		//Panels for tool buttons and address/port/ttl info
		panel=new Panel();
        panel.setLayout(new GridBagLayout());
		panelTool.add ("West",panel);

		panelStart.add("Center",startButton=new Button("Join conference session using selected media & tools:"));
	}

	public void addMedia(int tool, String media, String address, String port, String ttl, String key) {
		constrain(panel, (label[tool]=new Label(media)), 0, tool);
        constrain(panel, (check[tool]=new Checkbox("")),1, tool);
        check[tool].setState(true);
		constrain(panel, (choice[tool]=new Choice()), 2, tool);
        if (key.equals("")) {
		    constrain(panel, (text[tool]=new TextField("Address: "+address+"  Port: "+port+"  TTL: "+ttl,40)), 3, tool);
        } else {
		    constrain(panel, (text[tool]=new TextField("Address: "+address+"  Port: "+port+"  TTL: "+ttl+"   Key: "+key,40)), 3, tool);
        }
	}

	public void addTool(int tool, String toolName) {choice[tool].addItem(toolName);}

	public void addTitle(String title) {
		panelDesc.add ("Center", sessionTitle=new Label(title, Label.CENTER));
	}

	public void addDesc(String desc, String email, String phone, String url, long startTime, long endTime) {
        Date s, e;
        long adjust=Long.parseLong("2208988800");

	    panelDesc.add ("South", sessionDesc=new TextArea("", 4, 50, TextArea.SCROLLBARS_VERTICAL_ONLY));
        if (desc.length()!=0) sessionDesc.appendText("Session Description:\n"+desc+"\n");
        sessionDesc.appendText("\nSession Details:\n");
        if (email.length()!=0) sessionDesc.appendText("email address: "+email+"\n");
        if (phone.length()!=0) sessionDesc.appendText("phone number: "+phone+"\n");
        if (url.length()!=0) sessionDesc.appendText("web Address: "+url+"\n");
        if (startTime!=0) {
            startTime-=adjust;
            startTime*=1000;
            s=new Date(startTime);
            sessionDesc.appendText("\nSession Duration:\nstart: "+s.toGMTString()+"\n");
        }
        if (endTime!=0) {
            endTime-=adjust;
            endTime*=1000;
            e=new Date(endTime);
            sessionDesc.appendText("end: "+e.toGMTString());
        }
        sessionDesc.insertText("",0);
		sessionDesc.setEditable(false);
	}

    public String getTool(int tool) {return (choice[tool].getSelectedItem());}
    public boolean mediaSelected(int tool) {return (check[tool].getState());}
    public void deselectMedia(int tool) {check[tool].setState(false);}
    public void selectTool(int tool, String toolName) {choice[tool].select(toolName);}
    public void selectMedia(int tool) {check[tool].setState(true);}

    public void constrain(Container container, Component component, int gridx, int gridy) {
        GridBagConstraints c=new GridBagConstraints();
        c.gridx=gridx;
        c.gridy=gridy;
        c.gridwidth=1;
        c.gridheight=1;
        c.fill=GridBagConstraints.NONE;
        c.insets=new Insets(1,2,1,2);
        c.ipadx=2;
        c.ipady=2;
        c.anchor=GridBagConstraints.CENTER;
        c.weightx=0.0;
        c.weighty=0.0;
        ((GridBagLayout)container.getLayout()).setConstraints(component, c);
        container.add(component);
    }
}