/*
 * FILE:    ParseSDP.java
 * AUTHORS: Kristian Hasler
 * 
 * Copyright (c) 1999 University College London
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted, for non-commercial use only, provided
 * that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Computer Science
 *      Department at University College London
 * 4. Neither the name of the University nor of the Department may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 * Use of this software for commercial purposes is explicitly forbidden
 * unless prior written permission is obtained from the authors.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
 
import java.awt.*;
import java.applet.*;
import java.util.*;

class ParseSDP {
	public String[] address, port, protocol, ttl, format, media, key;
	private String phone="", email="", url="";
    private String desc="", title="", cryptFlag="";
	public int tool=-1;
    private long startTime, endTime;
    private separate SDP, times;
    private Stack flagStack;

	public ParseSDP(String SDPData, String Separator, GUI gui) {
		if ((SDPData.equals(null)) || (Separator.equals(null))) {
			System.out.println("Incorrect Parameters");
			return;
		}
		address=new String[4];
		ttl=new String[4];
		port=new String[4];
       	protocol=new String[4];
		format=new String[4];
		media=new String[4];
		key=new String[4];

		System.out.println ("SDP={"+SDPData+"}");
		System.out.println ("SDP length="+SDPData.length());
		System.out.println ("Separator={"+Separator+"}");
		System.out.println ("Parsing SDP...");
        SDP=new separate(SDPData, Separator);
		
		int i=0;
		// scan through lines
		while (i<SDP.numLines()) {
        	System.out.println (SDP.line(i));
			if (SDP.line(i).startsWith("s=")) {title=SDP.line(i).substring(2);}
			if (SDP.line(i).startsWith("e=")) {email=SDP.line(i).substring(2);}
			if (SDP.line(i).startsWith("p=")) {phone=SDP.line(i).substring(2);}
			if (SDP.line(i).startsWith("i=")) {desc=SDP.line(i).substring(2);}
			if (SDP.line(i).startsWith("u=")) {url=SDP.line(i).substring(2);}
			if (SDP.line(i).startsWith("t=")) {
                times=new separate(SDP.line(i).substring(2)," ");
                startTime=Long.parseLong(times.line(0));
                endTime=Long.parseLong(times.line(1));
            }

			if (SDP.line(i).startsWith("m=")) {
				StringTokenizer t1=new StringTokenizer(SDP.line(i), " ");
                if (tool<3) tool++;
				media[tool]=(t1.nextToken()).substring(2);
				port[tool]=t1.nextToken();
				protocol[tool]=t1.nextToken();
				format[tool]=t1.nextToken();
			}

			if (SDP.line(i).startsWith("c=IN IP")) {
				String addrport=SDP.line(i).substring(9);
				StringTokenizer t2=new StringTokenizer(addrport, "/");
				address[tool]=t2.nextToken();
				ttl[tool]=t2.nextToken();
				key[tool]="";
			}

			if (SDP.line(i).startsWith("k=clear:")) {
				key[tool]=SDP.line(i).substring(8);
			}
			i++;
		}

		gui.addTitle (title);
		gui.addDesc (desc, email, phone, url, startTime, endTime);
		for (i=0; i<tool+1; i++) {
			gui.addMedia(i, media[i], address[i], port[i], ttl[i], key[i]);
		}	
	}

	public Stack parsePlugins (int tool, String executable, separate plugins) {
		String keyflag, expanded="";
		int lineNo=0;
        boolean foundtool=false, foundmedia=false, foundformat=false;
        String osname=System.getProperty("os.name");
        separate flag;
        
        flagStack=new Stack();

        if (executable.equals("")) {
            System.out.println("No tool selected for media type: "+media[tool]);
        }

        while (lineNo<plugins.numLines()) {
            /* search plugins for executable/media type */
			if (plugins.line(lineNo).startsWith("media:")) {
                if ((plugins.line(lineNo).substring(6)).equals(media[tool])) foundmedia=true;
			}
			if (plugins.line(lineNo).startsWith("tool:") && foundmedia) {
                if ((plugins.line(lineNo).substring(5)).equals(executable)) foundtool=true;
                    else foundmedia=false;
			}			
			if (plugins.line(lineNo).startsWith("fmt:") && foundmedia && foundtool) {
                if ((plugins.line(lineNo).substring(4)).equals(format[tool])) foundformat=true;
			}
            
            if (plugins.line(lineNo).startsWith("cryptflag:")) {
                cryptFlag=plugins.line(lineNo).substring(10);
			}
            /* push on flagStack each flag option found */
            if (foundtool && foundmedia && foundformat) {        
                if (plugins.line(lineNo).startsWith("media:")) break;

			    if (plugins.line(lineNo).startsWith("flags:")) {
                    expanded=preExpand(plugins.line(lineNo).substring(6), tool);
                    flag=new separate(expanded, " ", "\"");
                    for (int i=0; i<flag.numLines(); i++) {
                        expanded=postExpand(flag.line(i), tool);

                        /* remove beginning and ending quotes from flags on unix */
                        if (osname.startsWith("Windows")) {
                            if (expanded.length()>0) flagStack.push(expanded);
                        } else {
                            if (expanded.length()>0) {
                               if (expanded.startsWith("\"") && expanded.endsWith("\"")) {
                                    flagStack.push(expanded.substring(1, (expanded.length()-1)));
                                } else flagStack.push(expanded);
                            }
                        }
                    }
			    }
            }
            lineNo++;
        }
		return flagStack;
	}

    /* expand known variables but don't remove unknown ones */
    public String preExpand(String flags, int tool) {
        int start, end;
        String keyword, flag;
        StringBuffer newFlag;
        newFlag=new StringBuffer();

        StringTokenizer t1=new StringTokenizer(flags, "$");
		while (t1.hasMoreTokens()) {
            flag=t1.nextToken();
            start=flag.indexOf("(");
            if (start!=-1) {
                end=flag.indexOf(")",1);
                keyword=flag.substring(1,end);

                if (keyword.equals("CRYPTKEY")) {
                   if ( (key[tool].length()!=0) && (cryptFlag.length()!=0) ) {
                        newFlag.append(cryptFlag+" "+key[tool]);
                        if ((end+1)!=flag.length()) newFlag.append(flag.substring(end+1)); 
                   } else newFlag.append("$"+flag);
                } else newFlag.append("$"+flag);
            } else newFlag.append(flag);
        }

        return (newFlag.toString());
    }

    /* expand rest of variables, removing unknown ones */
    public String postExpand(String flag, int tool) {
        int pos, start, end;
        String keyword;
        StringBuffer newFlag;

        do {
            pos=flag.indexOf("$");
            if (pos==-1) break;
            start=flag.indexOf("(",pos);
            end=flag.indexOf(")",start);
            keyword=flag.substring(start+1,end);
            System.out.println("expanding: "+keyword);
            newFlag=new StringBuffer();

            if (keyword.equals("TTL")) {
                newFlag.append(flag.substring(0, pos)+ttl[tool]);
            }
            else if (keyword.equals("SESSNAME")) {
                newFlag.append(flag.substring(0, pos)+"\""+title+"\"");
            }
            else if (keyword.equals("ADDRESS")) {
                newFlag.append(flag.substring(0, pos)+address[tool]);
            }
            else if (keyword.equals("PORT")) {
                newFlag.append(flag.substring(0, pos)+port[tool]);
            }
            else if (keyword.equals("CHAN")) {
                /* not supported yet - default flag */
                newFlag.append(flag.substring(0, pos)+"0");
            }
            else if (keyword.equals("PRIMARY")) {
                /* not supported yet - default flag */
                newFlag.append(flag.substring(0, pos)+"dvi");
            }
            else if (keyword.equals("SECONDARY")) {
                /* not supported yet - default flag */
                newFlag.append(flag.substring(0, pos)+"/dvi");
            }
            else {
                newFlag.append(flag.substring(0, pos));
            }
            if ((end+1)!=flag.length()) newFlag.append(flag.substring(end+1));
            flag=newFlag.toString();
        } while (true);

        return flag;
    }
}